@@ -1,9 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import json
 import os
 
@@ -13,9 +7,9 @@ from swagger_spec_validator.common import get_uri_from_file_path
 from tests import TESTS_BASE_PATH
 
 
-@pytest.fixture(scope='session')
+@pytest.fixture(scope="session")
 def petstore_contents():
-    with open(TESTS_BASE_PATH + '/data/v2.0/petstore.json') as f:
+    with open(TESTS_BASE_PATH + "/data/v2.0/petstore.json") as f:
         return f.read()
 
 
